import logging as log
import requests as req

from togglreports import utils


def get_workspaces(api_token: str) -> list[dict]:
    """
    """
    url = 'https://api.track.toggl.com/api/v8/workspaces'
    response = req.get(url, auth=(api_token, 'api_token'))

    try:
        retval = response.json()
    except Exception:
        log.error(f'Invalid response from Toggl: {response}')
        raise Exception('Invalid response from Toggl')

    log.debug(f'Workspace ids: {retval}')

    return retval


def get_me(api_token: str) -> dict:
    """
    """
    url = 'https://api.track.toggl.com/api/v8/me'
    response = req.get(url, auth=(api_token, 'api_token'))

    try:
        retval = response.json()
    except Exception:
        log.error(f'Invalid response from Toggl: {response}')
        raise Exception('Invalid response from Toggl')

    log.debug(f'Me: {retval}')

    return retval


def get_detailed_report(api_token: str, workspace_id: int, user_agent: str = 'reportApi', **kwargs) -> dict:
    """
    options:
    {
        'user_agent': str,
        'period': str,
        'start': str,
    }
    """
    url = 'https://api.track.toggl.com/reports/api/v2/details'

    # Get report horizon
    start_report_date, end_report_date = utils.get_period_start_end(period=kwargs.get('period', None), start=kwargs.get('start', None))

    # Make request
    payload = {
        'workspace_id': workspace_id,
        "page": 1,
        "user_agent": user_agent,
        "since": start_report_date,
        "until": end_report_date,

    }
    payload = {key: value for key, value in payload.items() if value is not None}

    response = req.get(url, auth=(api_token, 'api_token'), params=payload).json()
    time_entries = response.get('data')

    # If there are more entries than the page size, get the next page
    num_entries = response.get('total_count', 0)
    entries_per_page = response.get('per_page', 0)
    while (num_entries > entries_per_page):
        payload['page'] += 1
        response = req.get(url, auth=(api_token, 'api_token'), params=payload).json()
        time_entries += response.get('data')
        num_entries -= entries_per_page

    # If there is an error, raise an exception
    if response.get('error'):
        log.error(f'Error with the Toggl API. Message: {response.get("error").get("message")}, \
                Tip: {response.get("error").get("tip")}, Code: {response.get("error").get("code")}')
        raise Exception('Error returned from Toggl')

    log.debug(f'Toggl report: {time_entries}')

    return time_entries


if __name__ == '__main__':
    get_detailed_report('', 1)



from typing import Callable


class ExampleReport():
    """ Example report """

    _base_data: list[dict]
    _base_config: dict
    _config: dict

    def __init__(self, data: list[dict], base_config: dict, config: dict):
        self._base_data = data
        self._base_config = base_config
        self._config = config

    def export(self) -> None:
        print('export')


def register(register_function: Callable, name: str) -> None:
    """ Register plugin """
    register_function(name, ExampleReport)



from argparse import ArgumentParser

from togglreports import cli_parser, config
from togglreports.core import plugin_loader


class Application():
    _parser: ArgumentParser

    def run(self):
        # Load the plugins
        self._load_plugins()

        # Initialize the application
        self._initialize()

        # Parse the arguments
        cli_parser.process_arguments(self._parser)

    def _initialize(self):
        # Initialize the configuration if not present
        if not config.config_exists():
            config.init_config()

        self._parser = cli_parser.create_parser()
        if not self._parser:
            raise Exception('Parser not initialized')
            exit(1)

    def _load_plugins(self):
        plugin_loader.load_plugins()



import os
from configparser import ConfigParser

import togglreports.togglapi as togglapi
import togglreports.core.plugin_loader as pl

CONFIG_FILE = os.path.normpath(os.path.join(os.path.dirname(__file__), "./data", "config.ini"))


def init_config(report_section_prefix: str = 'reports'):
    ensure_config_path()
    config = ConfigParser()

    print("Initializing...")

    # Set user email
    while True:
        toggl_user_apitoken = input("api token: ")

        if not toggl_user_apitoken:
            print("Please enter an api token.")
            continue

        try:
            user_data = togglapi.get_me(toggl_user_apitoken).get('data')
        except Exception:
            print("Invalid api token.")
            continue

        break

    config['toggl.user'] = {
        'email': user_data.get('email'),
        'api_token': user_data.get('api_token'),
    }

    # Set workspace
    possible_workspaces = togglapi.get_workspaces(toggl_user_apitoken)
    if not possible_workspaces:
        print("No workspaces found.")
        exit(1)
    print("Select a workspace:")
    for i, workspace in enumerate(possible_workspaces):
        print(f"{i} - {workspace['name']}")
    while True:
        input_workspace_idx = input("workspace number (0): ")

        if (input_workspace_idx
            and (not input_workspace_idx.isdigit()
                 or int(input_workspace_idx) >= len(possible_workspaces))):
            print("Please enter a valid workspace number.")
            continue

        workspace_idx = 0 if not input_workspace_idx else int(input_workspace_idx)

        toggl_workspace = possible_workspaces[workspace_idx]
        break

    config['toggl.workspace'] = {
        'id': toggl_workspace.get('id'),
        'name': toggl_workspace.get('name'),
    }

    # Set optional output file configuration
    input_reports_name = input("report name (toggl_report): ")
    reports_name = 'toggl_report' if not input_reports_name else input_reports_name

    input_reports_add_date = input("use current date as prefix (y): ")
    reports_add_date = True if ((input_reports_add_date == 'y') or (not input_reports_add_date)) else False

    config['reports'] = {
        'name': reports_name,
        'add_date': reports_add_date,
    }

    # Set plugins config
    for plugin, plugin_config_list in pl.get_plugins_required_configuration().items():
        tmp_dict = {}
        for plugin_config in plugin_config_list:
            plugin_config_name = plugin_config.get('name')
            plugin_config_default = plugin_config.get('default', None)
            input_request_default = f' ({plugin_config_default})' if plugin_config_default else ''

            while True:
                input_data = input(f"[{plugin}] {plugin_config_name}{input_request_default}: ")

                if not input_data and not plugin_config_default:
                    print("Please enter a value.")
                    continue

                break

            tmp_dict[f'{plugin_config_name}'] = input_data if input_data else plugin_config_default

        config[f'{report_section_prefix}.{plugin}'] = tmp_dict

    # Write config file
    with open(CONFIG_FILE, 'w') as configfile:
        config.write(configfile)


def read_config() -> ConfigParser:
    config = ConfigParser()
    config.read(CONFIG_FILE)
    return config


def config_exists() -> bool:
    return os.path.exists(CONFIG_FILE)


def config_path_exists() -> bool:
    return os.path.exists(CONFIG_FILE[:-len(os.path.basename(CONFIG_FILE))])


def ensure_config_path() -> None:
    if not config_path_exists():
        os.mkdir(os.path.dirname(CONFIG_FILE))
    return None


def get_config_value(section: str, key: str) -> str:
    config = read_config()
    return config.get(section, key)


def get_config_section(section: str) -> dict:
    config = read_config()
    options = config.options(section)
    data = {}
    for option in options:
        data[option] = config.get(section, option)
    return data


def get_report_config(report: str, report_section_prefix: str = 'reports') -> dict:
    section = f'{report_section_prefix}.{report}'
    data = get_config_section(section)
    return data


if __name__ == '__main__':
    init_config()



import importlib
import os
import json
from typing import Callable

from togglreports.core import report_factory


PLUGIN_FILEPATH = os.path.normpath(os.path.join(os.path.dirname(__file__), "../data", "reports.json"))


class PluginInterface:
    """
    Represents a plugin interface
    """

    @staticmethod
    def register(register_function: Callable, name: str) -> None:
        """Register plugin"""


def import_plugin(name: str) -> PluginInterface:
    """Imports a module given a name."""
    return importlib.import_module(f"togglreports.plugins.{name}")  # type: ignore


def get_plugins() -> list[str]:
    """Returns a list of plugins."""
    with open(PLUGIN_FILEPATH) as file:
        data = json.load(file)
        return data["plugins"]


def get_plugins_required_configuration() -> dict[dict[str, str]]:
    """ Returns a list of all plugins required configuration """
    with open(PLUGIN_FILEPATH) as file:
        data = json.load(file)
        return data["config"]


def load_plugins() -> None:
    """Loads the plugins defined in the plugins list."""
    plugin_list = get_plugins()

    for plugin_name in plugin_list:
        plugin = import_plugin(plugin_name)
        plugin.register(report_factory.register, plugin_name)


if __name__ == '__main__':
    get_plugins_required_configuration()



from typing import Callable
import pandas as pd
import datetime as dt


class SGUReport():
    """
    Sgu report
    """

    _base_data: list[dict]
    _base_config: dict
    _config: dict

    _default_filename: str = 'SGU_report'
    _file_format: str = 'csv'
    _file_encoding: str = 'ansi'
    _separator: str = ';'

    _default_ignore_tag: str = '<IGNORE>'
    _max_num_chars: int = 50

    def __init__(self, data: list[dict], base_config: dict, config: dict):
        self._base_data = data
        self._base_config = base_config
        self._config = config

    def export(self) -> None:
        """ Export data to file """
        data = self._process(self._base_data)

        filename = self._base_config.get('name', self._default_filename)
        prefix = f'{dt.datetime.today().strftime("%Y%m%d")}_' if self._base_config.get('add_date', False) else ''

        pd.DataFrame.from_dict(data).to_csv(
            f'{prefix}{filename}.{self._file_format}',
            index=False,
            encoding=self._file_encoding,
            sep=self._separator
        )

    def _process(self, data: list[dict]) -> list[dict]:
        """ Process data to specific format """
        username = self._config.get('username', '')
        ignored_tag = self._config.get('ignored_tag', self._default_ignore_tag)
        default_tag = self._config.get('default_tag', '')
        processed_data = []

        for entry in data:
            # Check if entry is a valid entry
            if (len(entry['tags']) >= 1):
                if ignored_tag in entry['tags']:
                    continue
                categ = entry['tags'][0]
            else:
                categ = default_tag

            processed_data.append({
                'DATA': dt.datetime.strptime(entry['start'], "%Y-%m-%dT%H:%M:%S-03:00").strftime("%d/%m/%Y"),
                'PROJETO': entry['project'],
                'CATEGORIA': categ,
                'ATIVIDADE': entry['description'][:self._max_num_chars],
                'OPORTUNIDADE': '',
                'HORAS': str(float(entry['dur']) / 3600000).replace('.', ','),
                'USERNAME': username
            })

        return processed_data


def register(register_function: Callable, name: str) -> None:
    """ Register plugin """
    register_function(name, SGUReport)



""" Create a parser for the command line arguments """
from argparse import ArgumentParser

from togglreports.core import report_factory
from togglreports import build
from togglreports import config
from togglreports import utils


def process_arguments(parser: ArgumentParser) -> None:
    """ Process the parser """
    args = parser.parse_args()
    if args.cmd == 'build':
        build.build_report(args.type, period=args.period, start=args.start)
    elif args.cmd == 'config':
        config.init_config()
    else:
        parser.print_help()
        exit(1)


def create_parser() -> ArgumentParser:
    parser = ArgumentParser()

    subparsers = parser.add_subparsers(dest='cmd')

    _add_build_subparser(subparsers)
    _add_config_subparser(subparsers)

    return parser


def _add_build_subparser(subparser: ArgumentParser) -> None:
    """ Add the build subparser to the parser """

    parser = subparser.add_parser('build', help='Generate a report')
    parser.add_argument('type', choices=report_factory.PLUGINS, help='The report type to generate')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-p', '--period', choices=utils.PERIODS, help='The period to generate the report for')
    group.add_argument('-s', '--start', help='A date to start the report from. Expects YYYY-MM-DD format')
    parser.set_defaults(command='build')

    return None


def _add_config_subparser(subparser: ArgumentParser) -> None:
    """ Add the config subparser to the parser """

    parser = subparser.add_parser('config', help='Change the configuration')
    parser.set_defaults(command='config')

    return None



""" Factory for creating reports """

from typing import Callable

from togglreports.core.protocols import ReportPlugin


PLUGINS: dict[str, Callable[..., ReportPlugin]] = {}


def register(report_name: str, report_class: Callable[[], ReportPlugin]) -> None:
    """ Register report class """
    PLUGINS[report_name] = report_class


def unregister(report_name: str) -> None:
    """ Unregister report class """
    PLUGINS.pop(report_name, None)


def create_report(report_name: str, *args, **kwargs) -> None:
    """ Create report """
    try:
        report_class = PLUGINS[report_name]
    except KeyError as err:
        raise KeyError(f"Report {report_name} does not exist") from err

    report_class(*args, **kwargs).export()

    return None



import datetime as dt


PERIODS = ['thisweek', 'lastweek', 'thismonth', 'today']


def get_period_start_end(period: str = None, start: str = None, format: str = "%Y-%m-%d", reference: str = None) -> tuple[str, str]:
    """ Get datetime for start and end of a predetermined period.
    If no period is specified, get this weeks start and end date.
    """

    if period and start:
        raise Exception("Conflicting arguments: period and start")

    try:
        _ = dt.datetime.today().strftime(format)
    except ValueError:
        raise ValueError("Incorrect data format")

    if reference is not None:
        try:
            reference_date = dt.datetime.strptime(reference, format)
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY-MM-DD")
    else:
        reference_date = dt.datetime.today()

    if start is not None:
        try:
            start_report_date = dt.datetime.strptime(start, format)
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY-MM-DD")
        end_report_date = reference_date.replace(hour=23, minute=59, second=59)

    else:
        if period not in PERIODS:
            period = 'thisweek'

        reference_weekday = reference_date.weekday()

        if period == 'thisweek':
            start_report_date = reference_date - dt.timedelta(days=reference_weekday)
            end_report_date = start_report_date + dt.timedelta(days=6)
        elif period == 'lastweek':
            start_report_date = reference_date - dt.timedelta(days=reference_weekday + 7)
            end_report_date = start_report_date + dt.timedelta(days=6)
        elif period == 'thismonth':
            start_report_date = reference_date.replace(day=1)
            end_report_date = get_last_day_of_month(start_report_date)
        elif period == 'today':
            start_report_date = reference_date.replace(hour=0, minute=0, second=0)
            end_report_date = reference_date.replace(hour=23, minute=59, second=59)

    return start_report_date.strftime(format), end_report_date.strftime(format)


def get_last_day_of_month(this_month: dt.datetime) -> dt.datetime:
    """ Get the last day of the month for any day. """
    next_month = this_month.replace(day=28) + dt.timedelta(days=4)
    last_day_date = next_month - dt.timedelta(days=next_month.day)
    return last_day_date



from togglreports.core import report_factory
from togglreports import togglapi
from togglreports import config as cfg


def build_report(report_type: str, **kwargs):
    """ Build the report """
    data = get_report_data(**kwargs)
    config = cfg.get_report_config(report_type)
    base_config = cfg.get_config_section('reports')
    report_factory.create_report(report_type, data=data, config=config, base_config=base_config)


def get_report_data(**kwargs):
    """ Get the report data """
    api_token = cfg.get_config_value('toggl.user', 'api_token')
    workspace_id = cfg.get_config_value('toggl.workspace', 'id')

    data = togglapi.get_detailed_report(api_token=api_token, workspace_id=workspace_id, **kwargs)
    return data
